% Multi-resonance recursive fitting version 0.50
% Developed by Ruochen Lu (rlu10@illinois.edu), Tomas
% Manzaneque (T.ManzanequeGarcia@tudelft.nl), and Songbin Gong (songbin@illinois.edu)
% University of Illinois at Urbana-Champaign, 
% Illinois Integrated RF Microsystems Group
% http://ilirm.ece.illinois.edu/
% Details explained in our paper
% R. Lu, M. Li, Y. Yang, T. Manzaneque and S. Gong, "Accurate Extraction of Large Electromechanical Coupling in Piezoelectric MEMS Resonators," 
% in Journal of Microelectromechanical Systems. doi: 10.1109/JMEMS.2019.2892708
% Updated 2018/07/17

clear all;
close all;
clc;

%% define parameters here

filename='76.s1p';   %import s1p file, the format is "# Hz S  dB   R 50", Hz-dB-angle
frequency_range=[    %please adjust the import section if the touchstone files are saved differently
2000e6  5500e6 
3450e6  3600e6
3600e6  4000e6
3350e6  3400e6
4000e6  4100e6
4100e6  4500e6
2800e6  3200e6
5000e6  5200e6
];     

                            %first row is the total range for the fitting, 
                            %following rows are the fitting range for
                            %each mode, starting from high FoM to low FoM
                            %modes preferably, the code will calculate the
                            %total C0 and R0 in the end for faster
                            %convergence


% filename='4-7-q1774.s1p';  %example 1

% frequency_range=[
% 165e6   200e6
% 170e6   175e6
% 175e6   180e6
% 185e6   190e6
% 185e6   186.5e6
% 192e6   194e6
% ];  
     
% filename='76.s1p';    %example 2
% frequency_range=[
% 2000e6  5500e6 
% 3450e6  3600e6
% 3600e6  4000e6
% 3350e6  3400e6
% 4000e6  4100e6
% 4100e6  4500e6
% 2800e6  3200e6
% 5000e6  5200e6
% ];                

                            
flag_plot_recursive=2;      %0 if not plotting the recursive plot, 2 if plot per cycle, 1 if plot per step
flag_text_recursive=1;      %0 if not plotting the recursive information
flag_fit_floor=1;           %0 if not fitting floor, 1 if fit floor
recursive_number=10;         %maximum recursive number
number_recursive_individual_peak=100;   %maximum loop number for each single tone

%% import s1p file
fmt = repmat('%f', 1, 3);
fid = fopen(filename, 'rt');
datacell = textscan(fid, fmt, 'HeaderLines', 6, 'CollectOutput', 1);%defining number of comments at the beginning of the s1p file, please change this!!
fclose(fid);
data = datacell{1};
sdata=10.^(data(:,2)/20).*exp(j*data(:,3)/180*pi);  % if Hz S  dB   R 50
%sdata=data(:,2)+j*data(:,3);  % if Hz S  RI   R 50

freq0=data(:,1).';
admittance0=1/50*((1-sdata)./(1+sdata)).';  %50 ohm system
  %figure;
 % plot(freq0,db(abs(admittance0)));         %check input data
index0=freq0>frequency_range(1,1)&freq0<frequency_range(1,2);
freq=freq0(index0);                         %using the first row of the range definition
admittance=admittance0(index0);
%  figure;
%  plot(freq,db(abs(admittance)));           %check truncated data

deviation_list_db=[];                                %define deviation here, the default value is 1/N*sum(|dB(Yf)-dB(Ym)|+|phase(Yf)-phase(Ym)|)
deviation_list_phase=[]; 

figure(1)
plotdb=subplot(211);plot(freq,db(abs(admittance)),'r-');
xlabel('Frequency (Hz)','FontSize',12);
ylabel('Admittance (db)','FontSize',12);
plotphase=subplot(212);plot(freq,phase(admittance)/pi*180,'r-');
xlabel('Frequency (Hz)','FontSize',12);
ylabel('Phase (degree)','FontSize',12);

%% start initialization
[mode_number,tempn]=size(frequency_range);
[tempn,admittancelength]=size(admittance);
mode_number=mode_number-1;
temp_leftover=zeros(mode_number+1,admittancelength);
left_admittance=admittance;
v_list=zeros(mode_number,6); %6 is the number of parameters output from adjuste_impedancia function
v_floor=zeros(1,2);

%% recursive fitting
for h=1:recursive_number
    if flag_text_recursive
        disp(['cycle ' num2str(h) ' starts']);
    end
    for k=1:mode_number
        if flag_text_recursive
            disp(['peak ' num2str(k) ' starts']);
        end
        index_current=freq>frequency_range(k+1,1)&freq<frequency_range(k+1,2);
        left_admittance=admittance-sum(temp_leftover)+temp_leftover(k,:);
        
        [v,YY]=ajuste_impedancia_2(freq(index_current),left_admittance(index_current),number_recursive_individual_peak);        

        v_list(k,:)=v;
        temp_leftover_temp=admitancia_piezo2_noscale(v,freq);
        temp_leftover(k,:)=temp_leftover_temp(1,:)+1i.*temp_leftover_temp(2,:);
        if(flag_plot_recursive==1)
            hold(plotdb,'on');
            plot(plotdb,freq,db(abs(sum(temp_leftover))));
            hold(plotdb,'off');
            hold(plotphase,'on');
            plot(plotphase,freq,phase(sum(temp_leftover))/pi*180);
            hold(plotphase,'off');
            pause(0.1);
            
            N=length(db(abs((temp_leftover))));%evaluate standard deviation
            diff=abs(db(abs(sum(temp_leftover)))-db(abs((admittance))));
            deviation=sqrt(1/N*sum(diff.^2));
            disp(['deviation_db = ' num2str(deviation)]);
            deviation_list_db=[deviation_list_db deviation];      
            
            diff_phase=180/pi*min(abs(phase(sum(temp_leftover))-phase((admittance))),pi-abs(phase(sum(temp_leftover))-phase((admittance))));
            deviation_phase=sqrt(1/N*sum(diff_phase.^2));
            disp(['deviation_phase = ' num2str(deviation_phase)]);
            deviation_list_phase=[deviation_list_phase deviation_phase];     
        end
    end
      
    if flag_fit_floor
        if flag_text_recursive
            disp(['floor fitting starts']); %%fit floor
        end
        k=k+1;
        index_current=(freq>frequency_range(1,1))&(freq<frequency_range(1,2));
        left_admittance=admittance-sum(temp_leftover)+temp_leftover(k,:);
        [v_floor,YY]=ajuste_impedancia_floor(freq(index_current),left_admittance(index_current),number_recursive_individual_peak);        
        temp_leftover_temp=admitancia_piezo_floor_noscale(v_floor,freq);
        temp_leftover(k,:)=temp_leftover_temp(1,:)+1i.*temp_leftover_temp(2,:);
        if(flag_plot_recursive==1)
            hold(plotdb,'on');
            plot(plotdb,freq,db(abs(sum(temp_leftover))));
            hold(plotdb,'off');
            hold(plotphase,'on');
            plot(plotphase,freq,phase(sum(temp_leftover))/pi*180);
            hold(plotphase,'off');
            pause(0.1);

            N=length(db(abs((temp_leftover))));%evaluate standard deviation
            diff=abs(db(abs(sum(temp_leftover)))-db(abs((admittance))));
            deviation=sqrt(1/N*sum(diff.^2));
            disp(['deviation_db = ' num2str(deviation)]);
            deviation_list_db=[deviation_list_db deviation];      

            diff_phase=180/pi*min(abs(phase(sum(temp_leftover))-phase((admittance))),pi-abs(phase(sum(temp_leftover))-phase((admittance))));
            deviation_phase=sqrt(1/N*sum(diff_phase.^2));
            disp(['deviation_phase = ' num2str(deviation_phase)]);
            deviation_list_phase=[deviation_list_phase deviation_phase];    
        end
    end
    
    if(flag_plot_recursive==2)
        hold(plotdb,'on');
        plot(plotdb,freq,db(abs(sum(temp_leftover))));
        hold(plotdb,'off');
        hold(plotphase,'on');
        plot(plotphase,freq,phase(sum(temp_leftover))/pi*180);
        hold(plotphase,'off');          %%display after full cycle
        pause(0.1);       
    end
    
    N=length(db(abs((temp_leftover))));                            %evaluate standard deviation
    diff=abs(db(abs(sum(temp_leftover)))-db(abs((admittance))));
    deviation=sqrt(1/N*sum(diff.^2));
    disp(['deviation_db = ' num2str(deviation)]);
    deviation_list_db=[deviation_list_db deviation];      

    diff_phase=180/pi*min(abs(phase(sum(temp_leftover))-phase((admittance))),pi-abs(phase(sum(temp_leftover))-phase((admittance))));
    deviation_phase=sqrt(1/N*sum(diff_phase.^2));
    disp(['deviation_phase = ' num2str(deviation_phase)]);
    deviation_list_phase=[deviation_list_phase deviation_phase];    
end
%% plot and output
figure(2)
plotdb2=subplot(211);plot(freq,db(abs(admittance)),'r-',freq,db(abs(sum(temp_leftover))),'b-','LineWidth',2);
xlabel('Frequency (Hz)','FontSize',12);
ylabel('Admittance (db)','FontSize',12);
plotphase2=subplot(212);plot(freq,phase(admittance)/pi*180,'r-',freq,phase(sum(temp_leftover))/pi*180,'b-','LineWidth',2);
xlabel('Frequency (Hz)','FontSize',12);
ylabel('Phase (degree)','FontSize',12);

disp(['Summary'])
[tempa,tempb]=size(v_list);
if tempa>1
    temp_C0=sum(v_list)+real(v_floor(1));
else
    temp_C0=(v_list)+real(v_floor(1));
end
temp_R0=real(v_floor(2));

disp(['Ctotal = ' num2str(real(temp_C0(1))*1e15) ' fF']);
if flag_fit_floor
    disp(['R0 = ' num2str(temp_R0*1e-3) ' kohm']);
end
for k=1:mode_number
    if real(100*(pi^2)/8*v_list(k,4)/(temp_C0(1)))*sqrt(v_list(k,3)/v_list(k,4))/v_list(k,2)<0.01 %dont output if FoM<0.01
        continue
    end
    disp(['kt2_' num2str(k) ' = ' num2str(real(100*(pi^2)/8*v_list(k,4)/(temp_C0(1)))) ' %']);
    Q1=sqrt(v_list(k,3)/v_list(k,4))/v_list(k,2);
    disp(['Q' num2str(k) ' = ' num2str(Q1)]);
    fr1=1/2/pi/sqrt(v_list(k,3)*v_list(k,4));
    fprintf(['fr_' num2str(k) ' = ' num2str(fr1) ' Hz \n \n']);
end


figure;
subplot(211);plot([1:1:length(deviation_list_db)],deviation_list_db,'b-','LineWidth',2);
xlabel('Iteration Number','FontSize',12);
ylabel('Deviation Amplitude (dB)','FontSize',12);
subplot(212);plot([1:1:length(deviation_list_db)],deviation_list_phase,'b-','LineWidth',2);
xlabel('Iteration Number','FontSize',12);
ylabel('Deviation Phase (degree)','FontSize',12);